package census;

import ij.*;
import ij.process.*;
import ij.gui.*;
import ij.plugin.filter.Analyzer;
import ij.io.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import ij_ImageCanvasWithOverlay.ImageCanvasWithOverlay;


/**
 * CensusObject
 *
 * All censusObjects must share the following characteristics.
 *   1) Must have an ROI to represent them on the overlay (or modify the show() method).
 *   2) Must have a single set of (X,Y) co-ordinates as its representative point.
 *
 *
 * COPY IN
 *
 * @version    2.1 26 Jan 2009
 * @author     Peter Sebastian Masny
 */

public class CensusObject {
/*....................................................................
        Static variables
....................................................................*/
    public static final String IO_DELIMITER = ",";
    public static final String IO_DATA_SUFFIX = ".csv";
    public static final String[][] CHANNEL_COLOR = {
        {"None","Red","Green","Blue","Yellow","Cyan","Magenta"},
        {"None","R","G","B","Y","C","M"},
        {"000000","FF0000","00FF00","0000FF","FFFF00","00FFFF","FF00FF"} //for HTML
    };
    public static final Color[] COLOR_COLOR = {
        Color.black, Color.red, Color.green, Color.blue, Color.yellow, Color.cyan, Color.magenta
    };
    
    protected String breedName = "Undefined";
    
/*....................................................................
        Class variables
....................................................................*/
    public int x = -1; // error state until placed
    public int y;
    public int color;
    public ImagePlus img;
    Roi roi;
    public String annotation = "";
    
/*....................................................................
        Class constructors
....................................................................*/
    public CensusObject() {
        String ClassName = this.getClass().getName();
        this.breedName = ClassName.substring(7, ClassName.length()-5);
        //Typical classname: "census.SpotBreed"
        
    }
    
    
/*....................................................................
        Class methods
....................................................................*/
    
    public boolean find(int x, int y, ImagePlus img, int color) {
        // false when object fails sanity.
        this.x = x;
        this.y = y;
        this.img = img;
        this.color = color;
        return true;
    }
    
    
    public String asString() {
        // Meant for outputting to data files, such as .csv files.
        // Must always start with type, color
        // Should follwo with x,y
        String output = img.getOriginalFileInfo().directory;
        output += IO_DELIMITER + img.getOriginalFileInfo().fileName ;
        output += IO_DELIMITER + breedName ;
        output += IO_DELIMITER + CHANNEL_COLOR[0][color] ;
        output += IO_DELIMITER + x ;
        output += IO_DELIMITER + y ;
        output += IO_DELIMITER + annotation ;
        return output;
    }
    
    public boolean loadFromString(String s, ImagePlus img) {
        // false when object fails sanity
        // (Sanity checking would be good)
        // BreedName (values[2]) is used when constructing the object.
        
        String[] values = s.split(IO_DELIMITER, -1);
        for (int i=0;i<6;i++) { // Unsorted string search
            if (values[3].equals(CHANNEL_COLOR[0][i])) {
                this.color = i;
            }
        }
        this.x = new Integer(values[4]).intValue();
        this.y = new Integer(values[5]).intValue();
        this.annotation = values[6];
        if (this.color > 0) {
            if (find(x,y,img,color)) {
                return true;
            }
        }
        return false;
    }
    
    /*
     * This method returns specific information about the roi using IJ's measure pulgin.
     */
    public void IJmeasure() {
        Roi tempRoi = img.getRoi();
        img.setRoi(roi);
        Analyzer az = new Analyzer(img);
        az.run(img.getProcessor());
        //az.displayResults();
        
        // if image is RGB, also show the data for each channel.  Hopefully
        // someday ImageJ will have this feature built in.
        if (img.getType() == ImagePlus.COLOR_RGB) {
            int width = img.getWidth();
            int height = img.getHeight();
            byte[] R = new byte[width*height];
            byte[] G = new byte[width*height];
            byte[] B = new byte[width*height];
            ByteProcessor bpR = new ByteProcessor(width,height, R, LookUpTable.createGrayscaleColorModel(false));
            ByteProcessor bpG = new ByteProcessor(width,height, G, LookUpTable.createGrayscaleColorModel(false));
            ByteProcessor bpB = new ByteProcessor(width,height, B, LookUpTable.createGrayscaleColorModel(false));
            ((ColorProcessor)img.getProcessor()).getRGB(R, G, B);
            ImagePlus imgR = new ImagePlus("R",bpR);
            ImagePlus imgG = new ImagePlus("G",bpG);
            ImagePlus imgB = new ImagePlus("B",bpB);
 
            imgR.setRoi(roi);
            az = new Analyzer(imgR);
            az.run(imgR.getProcessor());
            imgR.close();
            imgG.setRoi(roi);
            az = new Analyzer(imgG);
            az.run(imgG.getProcessor());
            imgG.close();
            imgB.setRoi(roi);
            az = new Analyzer(imgB);
            az.run(imgB.getProcessor());
            imgB.close(); 
        }
        if (tempRoi == null) {
            img.killRoi();
        } else {
            img.setRoi(tempRoi);
        }
    }
    
    /*
     * This method colors the overlay (adding
     * not replacing other colors)
     */
    public void show(String type) {
        ImageCanvasWithOverlay icwo = (ImageCanvasWithOverlay)img.getCanvas();
        Color drawColor = COLOR_COLOR[color];
        if (type.equals("point")) {
            for(int ix=x-2;ix<x+3;ix++) {
                for(int iy=y-2;iy<y+3;iy++) {
                    icwo.overlaySetPixel(ix,iy,drawColor);
                }
            }
        } else if (type.equals("fill")) {
            Rectangle bound = roi.getBounds();
            for( int i =0; i< bound.width;i++) {
                for(int j=0; j< bound.height;j++) {
                    if (roi.contains(bound.x+i,bound.y+j)) {
                        icwo.overlaySetPixel(bound.x+i,bound.y+j,drawColor);
                    }
                }
            }
        } else if (type.equals("outline")) {
            Polygon poly = roi.getPolygon();
            for (int i = 0; i<poly.npoints-1;i++){
                icwo.overlaySetLine(poly.xpoints[i],poly.ypoints[i],poly.xpoints[i+1],poly.ypoints[i+1],drawColor);
            }
            icwo.overlaySetLine(poly.xpoints[poly.npoints-1],poly.ypoints[poly.npoints-1],poly.xpoints[0],poly.ypoints[0],drawColor);
        }
    }
    
    public static int[][] autoCandidates(ImageProcessor ip) {
        IJ.showMessage("No automatic finding action defined for this plugin");
        return null;
    }

    public void annotate() {
                String annoStr = "";
                boolean unsaneText = true;
                while  (unsaneText) {
                  GenericDialog gd = new GenericDialog("Annotation");
                  gd.addStringField("Annotation: ",annotation,20);
                  gd.addMessage("No commas or returns allowed");
                  gd.showDialog();
                  if (gd.wasCanceled()) return;
                  annoStr= gd.getNextString();
                  unsaneText = annoStr.contains(IO_DELIMITER);
                  if (unsaneText) IJ.error("Annotation phase cannot include \""+IO_DELIMITER+"\".");
                }
                this.annotation = annoStr;
    }
}